@@ -18,10 +18,11 @@ import pango
 from gobject import markup_escape_text
 from accerciser.plugin import ViewportPlugin
 from accerciser.icons import getIcon
-from accerciser.i18n import _, N_
+from accerciser.i18n import _, N_, DOMAIN
+from xml.dom import minidom
 
-GLADE_FILE = os.path.join(os.path.dirname(__file__), 
-                          'interface_view.glade')
+UI_FILE = os.path.join(os.path.dirname(__file__), 
+                       'interface_view.ui')
 
 class InterfaceViewer(ViewportPlugin):
   '''
@@ -43,34 +44,72 @@ class InterfaceViewer(ViewportPlugin):
     '''
     Intialize plugin.
     '''
-    glade_xml = gtk.glade.XML(GLADE_FILE, 'iface_view_frame')
-    frame = glade_xml.get_widget('iface_view_frame')
-    self.label_role = glade_xml.get_widget('label_role')
+    # HACK: Put all the callbacks in this class.
+    dom = minidom.parse(UI_FILE)
+    callbacks= set([signal.getAttribute('handler') \
+                      for signal in dom.getElementsByTagName('signal')])
+    del dom
+
+    ui_xml = gtk.Builder()
+    ui_xml.set_translation_domain(DOMAIN)
+    ui_xml.add_from_file(UI_FILE)
+    frame = ui_xml.get_object('iface_view_frame')
+    self.label_role = ui_xml.get_object('label_role')
     self.plugin_area.add(frame)
     self.sections = [
-      _SectionAccessible(glade_xml, self.node),
-      _SectionAction(glade_xml, self.node),
-      _SectionApplication(glade_xml, self.node),
-      _SectionComponent(glade_xml, self.node),
-      _SectionDocument(glade_xml, self.node),
-      _SectionHyperlink(glade_xml, self.node),
-      _SectionHypertext(glade_xml, self.node),
-      _SectionImage(glade_xml, self.node),
-      _SectionSelection(glade_xml, self.node),
-      _SectionStreamableContent(glade_xml, self.node),
-      _SectionTable(glade_xml, self.node),
-      _SectionText(glade_xml, self.node),
-      _SectionValue(glade_xml, self.node)]
+      _SectionAccessible(ui_xml, self.node),
+      _SectionAction(ui_xml, self.node),
+      _SectionApplication(ui_xml, self.node),
+      _SectionComponent(ui_xml, self.node),
+      _SectionDocument(ui_xml, self.node),
+      _SectionHyperlink(ui_xml, self.node),
+      _SectionHypertext(ui_xml, self.node),
+      _SectionImage(ui_xml, self.node),
+      _SectionSelection(ui_xml, self.node),
+      _SectionStreamableContent(ui_xml, self.node),
+      _SectionTable(ui_xml, self.node),
+      _SectionText(ui_xml, self.node),
+      _SectionValue(ui_xml, self.node)]
+
+    # HACK: Add callbacks to this class. 
+    for cb in callbacks:
+      for section in self.sections:
+        method = getattr(section, cb, None)
+        if not method: continue
+        setattr(self, cb, method)
+
+    ui_xml.connect_signals(self)    
 
     # Mark all expanders with no associated section classes as unimplemented
     implemented_sections = [obj.interface_name.lower() for obj in self.sections]
-    vbox_ifaces = glade_xml.get_widget('vbox_ifaces')
+    vbox_ifaces = ui_xml.get_object('vbox_ifaces')
     for expander in vbox_ifaces.get_children():
       iface_name = expander.name.replace('expander_', '')
       if iface_name not in implemented_sections:
-        section = _InterfaceSection(glade_xml, self.node, iface_name)
+        section = _InterfaceSection(ui_xml, self.node, iface_name)
         section.disable()
 
+    pyatspi.Registry.registerEventListener(
+        self.onAccNameChanged, 'object:property-change:accessible-name')
+
+  def onAccNameChanged(self, event):
+    '''
+    Listener for accessible name changes, if it is ours, change the name.
+
+    @param event: 'object:property-change:accessible-name' event.
+    @type acc: Accessibility.Event
+    '''
+    if event.source != self.node.acc: 
+      return
+
+    role = self.node.acc.getRoleName()
+    name = self.node.acc.name
+    if name:
+      role_name = '%s: %s' % (role, name)
+    else:
+      role_name = role
+    self.label_role.set_markup('<b>%s</b>' % markup_escape_text(role_name))
+      
   def onAccChanged(self, acc):
     '''
     Method that is invoked when the main accessible selection s changed.
@@ -108,12 +147,12 @@ class _InterfaceSection(object):
   @type event_listeners: list
   '''
   interface_name = None
-  def __init__(self, glade_xml, node, interface_name=None):
+  def __init__(self, ui_xml, node, interface_name=None):
     '''
     Initialize section object. and call init() for derived classes.
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     @param node: Application-wide node of selected accessible.
     @type node: L{Node}
     @param interface_name: Override default interface name.
@@ -122,17 +161,17 @@ class _InterfaceSection(object):
     self.interface_name = interface_name or self.interface_name
     self.node = node
     self.expander = \
-        glade_xml.get_widget('expander_%s' % self.interface_name.lower())
+        ui_xml.get_object('expander_%s' % self.interface_name.lower())
     self._setExpanderChildrenSensitive(False)
     self.event_listeners = []
-    self.init(glade_xml)
+    self.init(ui_xml)
 
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Abtract method for initializing section-specific code.
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
     pass
 
@@ -247,67 +286,38 @@ class _SectionAccessible(_InterfaceSection):
 
   interface_name = 'Accessible'
 
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Accessible interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)
     # Child count and description labels
-    self.child_count_label = glade_xml.get_widget('label_acc_child count')
-    self.desc_label = glade_xml.get_widget('label_acc_desc')
+    self.child_count_label = ui_xml.get_object('label_acc_child count')
+    self.desc_label = ui_xml.get_object('label_acc_desc')
 
     # configure states tree view
-    treeview = glade_xml.get_widget('states_view')
-    self.states_model = gtk.ListStore(str)
-    treeview.set_model(self.states_model)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=0)
-    treeview.append_column(tvc)
+    self.states_model = ui_xml.get_object('states_liststore')
 
     # configure relations tree view
-    self.relations_view = glade_xml.get_widget('relations_view')
-    self.relations_model = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
-    self.relations_view.set_model(self.relations_model)
-    crt = gtk.CellRendererText()
-    crp = gtk.CellRendererPixbuf()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crp, False)
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crp, pixbuf=0)
-    tvc.set_attributes(crt, text=1)
-    tvc.set_cell_data_func(crt, self._relationCellDataFunc)
-    tvc.set_cell_data_func(crp, self._relationCellDataFunc)
-    self.relations_view.append_column(tvc)
+    self.relations_view = ui_xml.get_object('relations_view')
+    self.relations_model = ui_xml.get_object('relations_treestore')
     # preset the different bg colors
     style = gtk.Style ()
-    self.header_bg = style.bg[gtk.STATE_NORMAL]
-    self.relation_bg = style.base[gtk.STATE_NORMAL]
+    self.header_bg = style.bg[gtk.STATE_NORMAL].to_string()
+    self.relation_bg = style.base[gtk.STATE_NORMAL].to_string()
+
     selection = self.relations_view.get_selection()
     selection.set_select_function(self._relationSelectFunc)
-    show_button = glade_xml.get_widget('button_relation_show')
+
+    show_button = ui_xml.get_object('button_relation_show')
     show_button.set_sensitive(self._isSelectedInView(selection))
     selection.connect('changed', self._onViewSelectionChanged, show_button)
 
     # configure accessible attributes tree view
-    treeview = glade_xml.get_widget('accattrib_view')
-    self.attr_model = gtk.ListStore(str, str)
-    treeview.set_model(self.attr_model)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=0)
-    treeview.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=1)
-    treeview.append_column(tvc)
+    self.attr_model = ui_xml.get_object('accattrib_liststore')
 
   def populateUI(self, acc):
     '''
@@ -338,10 +348,16 @@ class _SectionAccessible(_InterfaceSection):
     for relation in relations:
       r_type_name = repr(relation.getRelationType()).replace('RELATION_', '')
       r_type_name = r_type_name.replace('_', ' ').lower().capitalize()
-      iter = self.relations_model.append(None, [None, r_type_name, None])
+      iter = self.relations_model.append(
+          None, [None, 
+                 '<i>'+markup_escape_text(r_type_name)+'</i>', -1, 
+                 self.header_bg, False])
       for i in range(relation.getNTargets()):
         acc = relation.getTarget(i)
-        self.relations_model.append(iter, [getIcon(acc), acc.name, acc])
+        self.relations_model.append(
+            iter, [getIcon(acc), 
+                   markup_escape_text(acc.name), i, 
+                   self.relation_bg, True])
     self.relations_view.expand_all()
 
     self.registerEventListener(self._accEventState, 'object:state-changed')
@@ -354,34 +370,6 @@ class _SectionAccessible(_InterfaceSection):
     self.states_model.clear()
     self.attr_model.clear()
 
-  def _relationCellDataFunc(self, tvc, cellrenderer, model, iter):
-    '''
-    Make relation-type headers distinguishable by tweaking the visible styling.
-    
-    @param tvc: The given tree view column
-    @type tvc: gtk.TreeViewColumn
-    @param cellrenderer: The given cell renderer.
-    @type cellrenderer: gtk.CellRenderer
-    @param model: The relations data model.
-    @type model: gtk.TreeStore
-    @param iter: The current iter.
-    @type iter: gtk.TreeIter
-    '''
-    if len(model.get_path(iter)) == 1:
-      cellrenderer.set_property('cell-background-gdk', self.header_bg)
-      cellrenderer.set_property('mode', gtk.CELL_RENDERER_MODE_INERT)
-      if isinstance(cellrenderer, gtk.CellRendererText):
-        cellrenderer.set_property('style', pango.STYLE_ITALIC)
-      elif isinstance(cellrenderer, gtk.CellRendererPixbuf):
-        cellrenderer.set_property('visible', False)
-    else:
-      cellrenderer.set_property('cell-background-gdk', self.relation_bg)
-      cellrenderer.set_property('mode', gtk.CELL_RENDERER_MODE_ACTIVATABLE)
-      if isinstance(cellrenderer, gtk.CellRendererText):
-        cellrenderer.set_property('style', pango.STYLE_NORMAL)
-      elif isinstance(cellrenderer, gtk.CellRendererPixbuf):
-        cellrenderer.set_property('visible', True)
-
   def _relationSelectFunc(self, path):
     '''
     Make relation-type headers unselectable.
@@ -408,7 +396,9 @@ class _SectionAccessible(_InterfaceSection):
     selection = relations_view.get_selection()
     model, iter = selection.get_selected()
     if iter:
-      acc = model[iter][2]
+      path = model.get_path(iter)
+      relations = self.node.acc.getRelationSet()
+      acc = relations[path[0]].getTarget(model[iter][2])
       if acc:
         self.node.update(acc)
   
@@ -441,36 +431,19 @@ class _SectionAction(_InterfaceSection):
   @type action_selection: gtk.TreeSelection  
   '''
   interface_name = 'Action'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Action interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)    
     # configure actions tree view
-    treeview = glade_xml.get_widget('treeview_action')
-    self.actions_model = gtk.ListStore(int, str, str, str)
-    treeview.set_model(self.actions_model)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn(_('Name'))
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=1)
-    treeview.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn(_('Description'))
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=2)
-    treeview.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn(_('Key binding'))
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=3)
-    treeview.append_column(tvc)
+    treeview = ui_xml.get_object('treeview_action')
+    self.actions_model = treeview.get_model()
     self.action_selection = treeview.get_selection()
-    show_button = glade_xml.get_widget('button_action_do')
+    show_button = ui_xml.get_object('button_action_do')
     show_button.set_sensitive(self._isSelectedInView(self.action_selection))
     self.action_selection.connect('changed', 
                                   self._onViewSelectionChanged, show_button)
@@ -534,17 +507,17 @@ class _SectionApplication(_InterfaceSection):
   @type label_version: gtk.Label
   '''
   interface_name = 'Application'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Application interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    self.label_id = glade_xml.get_widget('label_app_id')
-    self.label_tk = glade_xml.get_widget('label_app_tk')
-    self.label_version = glade_xml.get_widget('label_app_version')
+    self.label_id = ui_xml.get_object('label_app_id')
+    self.label_tk = ui_xml.get_object('label_app_tk')
+    self.label_version = ui_xml.get_object('label_app_version')
   
   def populateUI(self, acc):
     '''
@@ -583,20 +556,20 @@ class _SectionComponent(_InterfaceSection):
   @type label_alpha: gtk.Label
   '''
   interface_name = 'Component'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Component interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    self.label_posabs = glade_xml.get_widget('absolute_position_label')
-    self.label_posrel = glade_xml.get_widget('relative_position_label')
-    self.label_size = glade_xml.get_widget('size_label')
-    self.label_layer = glade_xml.get_widget('layer_label')
-    self.label_zorder = glade_xml.get_widget('zorder_label')
-    self.label_alpha = glade_xml.get_widget('alpha_label')
+    self.label_posabs = ui_xml.get_object('absolute_position_label')
+    self.label_posrel = ui_xml.get_object('relative_position_label')
+    self.label_size = ui_xml.get_object('size_label')
+    self.label_layer = ui_xml.get_object('layer_label')
+    self.label_zorder = ui_xml.get_object('zorder_label')
+    self.label_alpha = ui_xml.get_object('alpha_label')
 
   def populateUI(self, acc):
     '''
@@ -651,29 +624,17 @@ class _SectionDocument(_InterfaceSection):
   @type label_locale: gtk.Label
   '''
 
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Document interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
     # configure document attributes tree view
-    treeview = glade_xml.get_widget('docattrib_view')
-    self.attr_model = gtk.ListStore(str, str)
-    treeview.set_model(self.attr_model)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=0)
-    treeview.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=1)
-    treeview.append_column(tvc)
-    self.label_locale = glade_xml.get_widget('label_doc_locale')
+    self.attr_model = ui_xml.get_object('docattrib_liststore')
+    self.label_locale = ui_xml.get_object('label_doc_locale')
 
   def populateUI(self, acc):
     '''
@@ -717,17 +678,16 @@ class _SectionHypertext(_InterfaceSection):
   @type links_model: gtk.ListStore
   '''
   interface_name = 'Hypertext'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Hypertext interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)
     # configure links tree view
-    treeview = glade_xml.get_widget('treeview_links')
+    treeview = ui_xml.get_object('treeview_links')
     # It's a treestore because of potential multiple anchors
     self.links_model = gtk.TreeStore(int, # Link index
                           str, # Name
@@ -766,7 +726,7 @@ class _SectionHypertext(_InterfaceSection):
     tvc.set_attributes(crt, text=5)
     treeview.append_column(tvc)    
     selection = treeview.get_selection()
-    show_button = glade_xml.get_widget('button_hypertext_show')
+    show_button = ui_xml.get_object('button_hypertext_show')
     show_button.set_sensitive(self._isSelectedInView(selection))
     selection.connect('changed', self._onViewSelectionChanged, show_button)
 
@@ -835,18 +795,18 @@ class _SectionImage(_InterfaceSection):
   '''
   interface_name = 'Image'
 
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Image interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    self.label_pos = glade_xml.get_widget('img_position_label')
-    self.label_size = glade_xml.get_widget('img_size_label')
-    self.label_locale = glade_xml.get_widget('img_locale_label')
-    self.label_desc = glade_xml.get_widget('img_locale_label')
+    self.label_pos = ui_xml.get_object('img_position_label')
+    self.label_size = ui_xml.get_object('img_size_label')
+    self.label_locale = ui_xml.get_object('img_locale_label')
+    self.label_desc = ui_xml.get_object('img_locale_label')
 
   def populateUI(self, acc):
     '''
@@ -884,37 +844,28 @@ class _SectionSelection(_InterfaceSection):
   '''
   interface_name = 'Selection'
 
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Selection interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)
     # configure selection tree view
-    treeview = glade_xml.get_widget('treeview_selection')
+    treeview = ui_xml.get_object('treeview_selection')
     self.sel_model = gtk.ListStore(gtk.gdk.Pixbuf, str, object)
     treeview.set_model(self.sel_model)
-    crt = gtk.CellRendererText()
-    crp = gtk.CellRendererPixbuf()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crp, False)
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crp, pixbuf=0)
-    tvc.set_attributes(crt, text=1)
-    treeview.append_column(tvc)
     # connect selection changed signal
     self.sel_selection = treeview.get_selection()
-    show_button = glade_xml.get_widget('button_select_clear')
+    show_button = ui_xml.get_object('button_select_clear')
     show_button.set_sensitive(self._isSelectedInView(self.sel_selection))
     self.sel_selection.connect('changed', 
                                self._onViewSelectionChanged, 
                                show_button)
     self.sel_selection.connect('changed', 
                                self._onSelectionSelected)
-    self.button_select_all = glade_xml.get_widget('button_select_all')
+    self.button_select_all = ui_xml.get_object('button_select_all')
 
   def populateUI(self, acc):
     '''
@@ -966,7 +917,7 @@ class _SectionSelection(_InterfaceSection):
       return
 
     model, paths = selection.get_selected_rows()
-    selected_children = [path[0] for path in paths]
+    selected_children = [model.get_value(model.get_iter(path), 2).getIndexInParent() for path in paths]
     
     for child_index in range(len(self.node.acc)):
       if child_index in selected_children:
@@ -1011,28 +962,16 @@ class _SectionStreamableContent(_InterfaceSection):
   '''
   interface_name = 'StreamableContent'
 
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the StreamableContent interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
     # configure streamable content tree view
-    treeview = glade_xml.get_widget('treeview_streams')
-    self.streams_model = gtk.ListStore(str, str)
-    treeview.set_model(self.streams_model)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn(_('Content type'))
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=0)
-    treeview.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn(_('URI'))
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=1)
-    treeview.append_column(tvc)
+    self.streams_model = ui_xml.get_object('streams_liststore')
 
   def populateUI(self, acc):
     '''
@@ -1084,28 +1023,27 @@ class _SectionTable(_InterfaceSection):
   @type cell_button: gtk.Button
   '''
   interface_name = 'Table'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Table interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)
-    self.selected_frame = glade_xml.get_widget('selected_cell_frame')
-    self.caption_label = glade_xml.get_widget('table_caption_label')
-    self.summary_label = glade_xml.get_widget('table_summary_label')
-    self.rows_label = glade_xml.get_widget('table_rows_label')
-    self.columns_label = glade_xml.get_widget('table_columns_label')
-    self.srows_label = glade_xml.get_widget('table_srows_label')
-    self.scolumns_label = glade_xml.get_widget('table_scolumns_label')
-    self.row_ext_label = glade_xml.get_widget('table_row_extents')
-    self.col_ext_label = glade_xml.get_widget('table_column_extents')
-    self.col_ext_label = glade_xml.get_widget('table_column_extents')
-    self.hrow_button = glade_xml.get_widget('table_hrow_button')
-    self.hcol_button = glade_xml.get_widget('table_hcol_button')
-    self.cell_button = glade_xml.get_widget('table_cell_button')
+    self.selected_frame = ui_xml.get_object('selected_cell_frame')
+    self.caption_label = ui_xml.get_object('table_caption_label')
+    self.summary_label = ui_xml.get_object('table_summary_label')
+    self.rows_label = ui_xml.get_object('table_rows_label')
+    self.columns_label = ui_xml.get_object('table_columns_label')
+    self.srows_label = ui_xml.get_object('table_srows_label')
+    self.scolumns_label = ui_xml.get_object('table_scolumns_label')
+    self.row_ext_label = ui_xml.get_object('table_row_extents')
+    self.col_ext_label = ui_xml.get_object('table_column_extents')
+    self.col_ext_label = ui_xml.get_object('table_column_extents')
+    self.hrow_button = ui_xml.get_object('table_hrow_button')
+    self.hcol_button = ui_xml.get_object('table_hcol_button')
+    self.cell_button = ui_xml.get_object('table_cell_button')
 
   def populateUI(self, acc):
     '''
@@ -1218,36 +1156,23 @@ class _SectionText(_InterfaceSection):
   @type outgoing_calls: dictionary
   '''
   interface_name = 'Text'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Text interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)
     # configure text attribute tree view
-    treeview = glade_xml.get_widget('treeview_text_attr')
-    self.attr_model = gtk.ListStore(str, str)
-    treeview.set_model(self.attr_model)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=0)
-    treeview.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn()
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=1)
-    treeview.append_column(tvc)
+    self.attr_model = ui_xml.get_object('textattrib_liststore')
 
-    self.offset_spin = glade_xml.get_widget('spinbutton_text_offset')
-    self.text_view = glade_xml.get_widget('textview_text')
+    self.offset_spin = ui_xml.get_object('spinbutton_text_offset')
+    self.text_view = ui_xml.get_object('textview_text')
     self.text_buffer = self.text_view.get_buffer()
-    self.toggle_defaults = glade_xml.get_widget('checkbutton_text_defaults')
-    self.label_start = glade_xml.get_widget('label_text_attr_start')
-    self.label_end = glade_xml.get_widget('label_text_attr_end')
+    self.toggle_defaults = ui_xml.get_object('checkbutton_text_defaults')
+    self.label_start = ui_xml.get_object('label_text_attr_start')
+    self.label_end = ui_xml.get_object('label_text_attr_end')
 
     self._text_insert_handler = 0
     self._text_delete_handler = 0
@@ -1302,7 +1227,7 @@ class _SectionText(_InterfaceSection):
     label_text = expander_label.get_label()
     label_text = label_text.replace(_('<i>(Editable)</i>'),'')
     label_text = label_text.strip(' ')
-    if eti:
+    if eti and acc.getState().contains(pyatspi.STATE_EDITABLE):
       label_text += ' ' + _('<i>(Editable)</i>')
       self.text_view.set_editable(True)
     else:
@@ -1499,7 +1424,7 @@ class _SectionText(_InterfaceSection):
         self.text_buffer.handler_unblock(self._text_insert_handler)
            
       elif event.type.minor == 'delete':
-        call = (event.detail1, event.detail2)
+        call = (event.detail1, event.detail1 + event.detail2)
         if self.outgoing_calls['itext_delete'].isCached(call):
           return
         text_iter_end = \
@@ -1605,19 +1530,18 @@ class _SectionValue(_InterfaceSection):
   @type label_inc: gtk.Label
   '''
   interface_name = 'Value'
-  def init(self, glade_xml):
+  def init(self, ui_xml):
     '''
     Initialization that is specific to the Value interface 
     (construct data models, connect signals to callbacks, etc.)
     
-    @param glade_xml: Interface viewer glade xml.
-    @type glade_xml: gtk.glade.XML
+    @param ui_xml: Interface viewer glade xml.
+    @type ui_xml: gtk.glade.XML
     '''
-    glade_xml.signal_autoconnect(self)
-    self.spinbutton = glade_xml.get_widget('spinbutton_value')
-    self.label_max = glade_xml.get_widget('label_value_max')
-    self.label_min = glade_xml.get_widget('label_value_min')
-    self.label_inc = glade_xml.get_widget('label_value_inc')
+    self.spinbutton = ui_xml.get_object('spinbutton_value')
+    self.label_max = ui_xml.get_object('label_value_max')
+    self.label_min = ui_xml.get_object('label_value_min')
+    self.label_inc = ui_xml.get_object('label_value_inc')
     self.registerEventListener(self._accEventValue, 
                                'object:value-changed')
     
@@ -1641,7 +1565,12 @@ class _SectionValue(_InterfaceSection):
       digits += 1
       minimumIncrement *= 10
 
+    # Calling set_range will clamp the value of spinbutton to the allowable
+    # range, causing us to try to set the value of the accessible when we
+    # really shouldn't.
+    self.ignore_value_changes = True
     self.spinbutton.set_range(vi.minimumValue, vi.maximumValue)
+    self.ignore_value_changes = False
     self.spinbutton.set_value(vi.currentValue)
     self.spinbutton.set_digits(digits)
    
@@ -1652,6 +1581,7 @@ class _SectionValue(_InterfaceSection):
     @param spinner: The Value spinner
     @type spinner: gtk.SpinButton
     '''
+    if self.ignore_value_changes: return
     vi = self.node.acc.queryValue()
     vi.currentValue = spinner.get_value()
 
